define(function(){
	'use strict';

/*
	leapMinX: -200,
	leapMaxX: 200,
	leapMinY: 100,
	leapMaxY: 600,
	leapMinZ: -180,
	leapMaxZ: 180,

	x -411.312 417.185
	y 25.9691 593.615
	z -291.756 432.686 
*/
	function Renderer(){
		var self = this;

	    this.camera = new THREE.PerspectiveCamera( 75, window.innerWidth / window.innerHeight, 1, 10000 );
        this.camera.position.y = 700;
        this.camera.position.z = 800;

        this.scene = new Physijs.Scene();//new THREE.Scene();
        this.scene.setGravity(new THREE.Vector3( 0, -600, 0 ));

        this.addPlayArea();

        this.camera.target = new THREE.Vector3(0,0,0);
		this.camera.lookAt(this.camera.target);

        this.fingers = [];
        this.numVisibleFingers = 0;
        _(10).times(function(){
        	self.fingers.push(self.makeFinger());
        });

        this.hands = [];
        this.numVisibleHands = 0;
        _(2).times(function(){
        	self.hands.push(self.makeHand());
        });

        // _(this.fingers.concat(this.hands)).each(function(mesh){
        // 	self.scene.add(mesh);
        // });

        this.renderer = new THREE.WebGLRenderer({ antialias: true, clearColor: 0xffffff });
        this.renderer.setSize(window.innerWidth, window.innerHeight);
        this.renderer.shadowMapEnabled = true;

        this.addLights();

        this.boxes = [];
        function makeToy(){
        	self.addToy(Math.floor(Math.random()*150)+50);
        	setTimeout(makeToy,3000);
        }
        makeToy();
        // _(10).times(function(){
        // 	self.addToy(Math.floor(Math.random()*150)+50);
        // });
	}

	Renderer.prototype.addLights = function(){
		var ambientLight = new THREE.AmbientLight( 0x3f2806 );
		this.scene.add( ambientLight );

		var pointLight = new THREE.PointLight( 0xffaa00, 1, 5000 );
		pointLight.position.set( 0, 0, 0 );
		// this.scene.add( pointLight );

		var sunLight = new THREE.SpotLight( 0xffffff, 1, 0, Math.PI/2, 1 );
		sunLight.position.set( 1000, 2000, 1000 );

		sunLight.castShadow = true;
		sunLight.shadowCameraNear = 1000;
		sunLight.shadowDarkness = 0.6;
		this.scene.add(sunLight);
	};

	Renderer.prototype.addToy = function(size){
		var box = this.makeToy(size,{color: Math.random() < 0.5 ? 0X3333ff : 0xff3333, wireframe: false, castShadow: true});

        box.position.y = 700 + Math.floor(Math.random()*1000);
        box.position.x = Math.floor((Math.random()*400)-200);
        box.position.z = Math.floor((Math.random()*200)-100);

        box.rotation.y = Math.random() * Math.PI;
        box.rotation.x = Math.random() * Math.PI;
        box.rotation.z = Math.random() * Math.PI;

		box.castShadow = true;
		box.receiveShadow  = false;

		this.boxes.push(box);
		this.scene.add(box);
	};

	Renderer.prototype.makeToy = function(size, materialOptions, mass){
		// if(Math.random()<0.2){
			return new Physijs.SphereMesh(
				new THREE.SphereGeometry(size/2, 16, 16),//size, size),
				new THREE.MeshLambertMaterial(materialOptions),
				mass
			);
		// }
		return new Physijs.BoxMesh(
			new THREE.CubeGeometry(size, size, size),
			new THREE.MeshLambertMaterial(materialOptions),
			mass
		);
	};

	Renderer.prototype.makePlane = function(sizeX, sizeZ, materialOptions, mass){
		return new Physijs.BoxMesh(
			new THREE.CubeGeometry(sizeX, 1, sizeZ),
			new THREE.MeshLambertMaterial(materialOptions),
			mass
		);
	};

	Renderer.prototype.addPlayArea = function(){
        var floor = this.makePlane(800, 800, {color: 0xCCCCCC, receiveShadow: true, map: THREE.ImageUtils.loadTexture('checkerboard.png')}, 0),
        	walls = [
        		this.makePlane(400, 800, {color: 0xFFFFFF}, 0),
        		this.makePlane(800, 400, {color: 0xFFFFFF}, 0),
        		this.makePlane(400, 800, {color: 0xFFFFFF}, 0)
        	],
        	self = this;

        floor.position.y = 75;
		floor.receiveShadow = true;

		walls[0].position.x = -400;
		walls[0].rotation.z = Math.PI/2;

		walls[1].position.x = 0;
		walls[1].position.z = -400;
		walls[1].rotation.x = Math.PI/2;

		walls[2].position.x = 400;
		walls[2].rotation.z = Math.PI/2;

		this.scene.add(floor);
		_(walls).each(function(wall){
			wall.position.y = 275;
			self.scene.add(wall);
		});
	};

	Renderer.prototype.getDom = function(){
		return this.renderer.domElement;
	};

	Renderer.prototype.cull = function(){
		var self = this,
			newBoxes;

		_(this.fingers.concat(this.hands)).each(function(mesh){
			for(var i = self.scene.children.length - 1, obj; i >= 0; i--){
				obj = self.scene.children[i];
				if(obj === mesh){
					self.scene.remove(mesh);
				}
			}
		});
		this.boxes = _(this.boxes).filter(function(box,i){
			return box.y > 0;
		});
		this.numVisibleFingers = 0;
		this.numVisibleHands = 0;
	};

	Renderer.prototype.render = function(){
		this.scene.simulate();
		this.renderer.render(this.scene, this.camera);
		this.cull();
	};

	Renderer.prototype.makeFinger = function(){
		var geom = new THREE.CubeGeometry(15,10,40),
			mat = new THREE.MeshLambertMaterial({color: 0x00ff00}),
			mesh = new Physijs.BoxMesh(geom, mat, 0);//new THREE.Mesh(geom, mat);//Physijs.BoxMesh(geom, mat, 0);

		mesh.castShadow = true;
		mesh.__dirtyPosition = true;
		return mesh;
	};

	Renderer.prototype.makeHand = function(){
		var geom = new THREE.CubeGeometry(80,10,120),
			mat = new THREE.MeshLambertMaterial({color: 0x00ff00}),
			mesh = new Physijs.BoxMesh(geom, mat, 0);

		mesh.castShadow = true;
		mesh.__dirtyPosition = true;
		return mesh;
	};

	Renderer.prototype.showFinger = function(tipPosition){
		var fingerMesh = this.fingers[this.numVisibleFingers];

		if(!fingerMesh || tipPosition[1]<100){ return; }
		fingerMesh.position.x = (tipPosition[0] | 0) * 1.5;
		fingerMesh.position.y = tipPosition[1] | 0;
		fingerMesh.position.z = (tipPosition[2] | 0) * 1.7;
		
		this.scene.add(fingerMesh);

		this.numVisibleFingers++;
	};

	Renderer.prototype.showHand = function(handPosition, rotation){
		var handMesh = this.hands[this.numVisibleHands];

		if(!handMesh || handPosition[1]<100){ return; }
		handMesh.position.x = (handPosition[0] | 0) * 1.5;
		handMesh.position.y = handPosition[1] | 0;
		handMesh.position.z = (handPosition[2] | 0) * 1.7;

		//handMesh.matrix.setRotationFromEuler(rotation);
		// handMesh.rotation.setEulerFromRotationMatrix(
		// 	new THREE.Matrix3(
		// 		rotation[0][0], rotation[0][1], rotation[0][2],
		// 		rotation[1][0], rotation[1][1], rotation[1][2],
		// 		rotation[2][0], rotation[2][1], rotation[2][2]
		// 	)
		// );
		// handMesh.matrix.rotation = new THREE.Matrix3(
		// 		rotation[0][0], rotation[0][1], rotation[0][2],
		// 		rotation[1][0], rotation[1][1], rotation[1][2],
		// 		rotation[2][0], rotation[2][1], rotation[2][2]
		// 	);

//Vector3.setEulerFromRotationMatrix( matrix )
		var vec = new THREE.Vector3(),
			rotationMatrix = new THREE.Matrix3(
				rotation[0][0], rotation[0][1], rotation[0][2],
				rotation[1][0], rotation[1][1], rotation[1][2],
				rotation[2][0], rotation[2][1], rotation[2][2]
			);

		vec.setEulerFromRotationMatrix(rotationMatrix);
console.log(rotationMatrix);
//		debugger;

		// handMesh.rotation.y = rotation[0][0];
		// handMesh.rotation.y = rotation[1][0];
		// handMesh.rotation.z = rotation[2][0];

		this.scene.add(handMesh);

		this.numVisibleHands++;
	};

	return Renderer;
});